@@ -27,6 +27,11 @@
 --
 --
 -- $Log: file.c,v $
+-- Revision 1.75  2007/08/01 14:29:20  jsedwards
+-- Added code to recalculate data checksum when File object is updated and to
+-- make sure an updated private File class defintion exists before reading any
+-- old File objects in add_revision function.
+--
 -- Revision 1.74  2007/07/30 18:49:31  jsedwards
 -- Changed to NOT attempt to create the updated File private class definition
 -- when an old object is updated.
@@ -1412,6 +1417,7 @@ ObjCreateResult nwos_add_new_revision_of_file(char* path, ObjRef* ref)
     size_t length;
     struct stat stat_struct;
     ObjRef assoc_class_ref;
+    ObjRef file_class_ref;
     ObjRef path_ref;
     ObjRef file_ref;
     EveryObject header;
@@ -1475,6 +1481,11 @@ ObjCreateResult nwos_add_new_revision_of_file(char* path, ObjRef* ref)
     ref_list = nwos_malloc_reference_list(&header.object.references);
     num_refs = ref_list->common_header.num_refs;
 
+    /* Must make sure the File class is up to date before searching for the old file, */
+    /* in case it is of the old flavor (it has to be updated before rewritting it.    */
+
+    nwos_find_or_create_private_class_definition("FILE", &file_class_ref);
+
     assert(nwos_find_private_class_definition("PATH AND FILE ASSOCIATION", &assoc_class_ref));
 
     memset(&old_file_obj, 0, sizeof(old_file_obj));  /* zero it out */
@@ -1491,6 +1502,8 @@ ObjCreateResult nwos_add_new_revision_of_file(char* path, ObjRef* ref)
 
 	    nwos_read_object_from_disk(&assoc_obj.file, &new_file_obj, sizeof(new_file_obj));
 
+	    assert(is_same_object(&new_file_obj.header.common.class_definition, &file_class_ref));
+
 	    if (is_void_reference(&new_file_obj.header.object.next_version))
 	    {
 		if (!is_void_reference(&old_file_obj.header.common.id))  /* already found a file */
@@ -2925,6 +2938,8 @@ void nwos_update_file_001_object_to_current(void* object, size_t size)
     void_reference(&file_obj_ptr->sha256sum);
     copy_reference(&file_obj_ptr->media,      &file_001_obj.media);
     copy_reference(&file_obj_ptr->block_list, &file_001_obj.block_list);
+
+    nwos_crc32_calculate((uint8*) &file_obj_ptr->size[0], sizeof(C_struct_File) - sizeof(EveryObject), file_obj_ptr->header.common.data_chksum);
 }
 
 
