@@ -1,23 +1,72 @@
-/*             This file is part of the New World OS project
+/*
+--             This file is part of the New World OS project
 --                 Copyright (C) 2004-2007  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
 --
--- NWOS is free software;  you can redistribute it and/or modify it under the
--- terms of the GNU General Public License  as published by the Free Software
--- Foundation; either version 2, or (at your option) any later version.  This
--- software is distributed with the hope that it will be useful,  but WITHOUT
--- ANY WARRANTY;  without  even the  implied warranty  of MERCHANTABILITY  or
--- FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License for
--- more  details.  You should have received a copy  of the GNU General Public
--- License along with this package;  see the file LICENSE.  If not, write to:
+--   This program is free software: you can redistribute it and/or modify
+--   it under the terms of the GNU General Public License as published by
+--   the Free Software Foundation, either version 3 of the License, or
+--   (at your option) any later version.
+--
+--   This program is distributed in the hope that it will be useful,
+--   but WITHOUT ANY WARRANTY; without even the implied warranty of
+--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+--   GNU General Public License for more details.
+--
+--   You should have received a copy of the GNU General Public License
+--   along with this program, in the file LICENSE.  If not, see 
+--   <http://www.gnu.org/licenses/>.
+--
+--   You can also contact me via paper mail at:
+--
+--      QRW Software
+--      P.O. Box 27511
+--      Salt Lake City, UT 84127-0511, USA.
 --
---      Free Software Foundation, Inc.
---      59 Temple Place - Suite 330
---      Boston, MA 02111-1307, USA.
 --
 -- $Log: objectify.c,v $
+-- Revision 1.128  2007/07/30 18:35:02  jsedwards
+-- Added code to save the old file class in case an updated private class
+-- definition has not been created yet.
+--
+-- Revision 1.127  2007/07/29 21:47:08  jsedwards
+-- Fixed bug in assert that is not used anymore (just in case it is ever
+-- active again).
+--
+-- Revision 1.126  2007/07/29 19:43:09  jsedwards
+-- Removed variable that is now unused because of eliminating assert in
+-- previous revision.
+--
+-- Revision 1.125  2007/07/29 19:39:24  jsedwards
+-- Bug #1754741 - ifdef'd out assert to check that block was zero before
+-- writing to it.  This was before the bit map was used and now we assume
+-- the bit map is correct and we can write the block even if it is non-zero.
+--
+-- Revision 1.124  2007/07/29 18:25:54  jsedwards
+-- Added boolean variable to get_file_001_reference so that if it was called
+-- recursively it would simply return instead of overflowing the stack.
+--
+-- Revision 1.123  2007/07/29 03:48:17  jsedwards
+-- Add get_file_class_reference function to get the most up to date File class
+-- reference and make sure get_object_class returns it if the object is an
+-- older file object.
+--
+-- Revision 1.122  2007/07/28 15:20:47  jsedwards
+-- Fixed the case in finding the File 001 class where no private File objects
+-- have been created and a private file class doesn't exist.  Also fixed the
+-- order saving the reference in the loop so it saved it before reading next.
+--
+-- Revision 1.121  2007/07/28 04:23:31  jsedwards
+-- Add code to check for old File object and update it to new if found.
+--
+-- Revision 1.120  2007/07/06 15:02:34  jsedwards
+-- Moved nwos_next_public_ref and nwos_generate_new_public_id to disk_io.c.
+--
+-- Revision 1.119  2007/07/01 19:44:12  jsedwards
+-- Upgrade to GPLv3.
+--
 -- Revision 1.118  2007/06/26 14:17:25  jsedwards
 -- Move generation of random id functions to disk_io.c.
 --
@@ -690,11 +739,11 @@
 /* Global variables... BAD!... need to fix */
 
 bool verbose = false;
+bool nwos_ready = false;
 
 ObjRef nwos_public_class_definition_class_ref;
 ObjRef nwos_private_class_definition_class_ref;
 ObjRef nwos_reference_list_class_ref;
-ObjRef nwos_next_public_ref;
 
 static Encryption_Level encryption_level = Encryption_High;
 
@@ -1011,6 +1060,8 @@ void nwos_set_root_object(ObjRef* ref)
 
     assert(nwos_find_public_class_definition("REFERENCE LIST", &verify_ref));
     assert(is_same_object(&verify_ref, &nwos_reference_list_class_ref));
+
+    nwos_ready = true;
 }
 
 
@@ -1018,6 +1069,8 @@ void nwos_terminate_objectify()
 {
     nwos_log("nwos_terminate_objectify():");
 
+    nwos_ready = false;
+
     nwos_flush_dirty_ref_lists();  /* flush any reference lists that are dirty out to disk */
 
     nwos_terminate_disk_io();
@@ -1154,37 +1207,6 @@ void nwos_generate_new_id(ObjRef* ref)
 }
 
 
-#ifdef PUBLIC_MODE
-void nwos_generate_new_public_id(ObjRef* ref)
-{
-    int i;
-    int acc;
-    uint8 diff[4];
-
-    copy_reference(ref, &nwos_next_public_ref);
-
-    for (i = sizeof(nwos_next_public_ref)-1; i > -1; i--)
-    {
-	nwos_next_public_ref.id[i]++;
-	if (nwos_next_public_ref.id[i] != 0) break;
-    }
-
-    acc = (int)nwos_next_public_ref.id[3] - (int)ref->id[3];
-    diff[3] = acc;
-    acc = (int)nwos_next_public_ref.id[2] - (int)ref->id[2] + (acc >> 8);
-    diff[2] = acc;
-    acc = (int)nwos_next_public_ref.id[1] - (int)ref->id[1] + (acc >> 8);
-    diff[1] = acc;
-    acc = (int)nwos_next_public_ref.id[0] - (int)ref->id[0] + (acc >> 8);
-    diff[0] = acc;
-
-    assert(diff[0] == 0 && diff[1] == 0 && diff[2] == 0 && diff[3] == 1);
-
-    nwos_used_public_blocks++;
-}
-#endif
-
-
 /*************************/
 /* Verification routines */
 /*************************/
@@ -1358,6 +1380,83 @@ void nwos_read_object_from_disk_and_decrypt(ObjRef* ref, void* object, size_t si
 }
 
 
+#ifndef USE_PREDEFINED_STRUCTS
+static ObjRef* get_file_class_reference()
+{
+    static ObjRef result;
+
+    if (is_void_reference(&result))
+    {
+        /* can't do anything until everything is initialized */
+	if (nwos_ready)
+	{
+#ifdef PUBLIC_MODE
+	    nwos_find_public_class_definition("FILE", &result);
+#else
+	    /* in case there isn't a private one, make non-void so won't rerun this each time */
+	    if (!nwos_find_private_class_definition("FILE", &result))
+	    {
+		result.word = 0xffffffff;
+	    }
+#endif
+	}
+    }
+
+    return &result;
+}
+
+
+static ObjRef* get_file_001_reference()
+{
+    static ObjRef result;
+    static bool inside_already = false;
+    ObjRef class_ref;
+    EveryObject header;
+
+    if (is_void_reference(&result))
+    {
+        /* can't do anything until everything is initialized */
+	if (nwos_ready && !inside_already)
+	{
+	    inside_already = true;   /* in case we get called recursively */
+
+#ifdef PUBLIC_MODE
+	    nwos_find_public_class_definition("FILE", &class_ref);
+	    nwos_read_object_headers_from_disk(&class_ref, &header);
+	    assert(!is_void_reference(&header.object.prev_version));  /* there has to be at least one */
+#else
+	    /* in case there isn't an old one, make something non-void so won't rerun this each time */
+	    result.word = 0xffffffff;
+
+	    if (nwos_find_private_class_definition("FILE", &class_ref))
+	    {
+		nwos_read_object_headers_from_disk(&class_ref, &header);
+
+		if (is_void_reference(&header.object.prev_version))  /* updated class not created yet */
+		{
+		    copy_reference(&result, &class_ref);
+		}
+	    }
+	    else
+	    {
+		void_reference(&header.object.prev_version);
+	    }
+#endif
+	    while(!is_void_reference(&header.object.prev_version))
+	    {
+		copy_reference(&result, &header.object.prev_version);
+		nwos_read_object_headers_from_disk(&result, &header);
+	    }
+
+	    inside_already = false;
+	}
+    }
+
+    return &result;
+}
+#endif
+
+
 #define SIZE_EXTRA_BLOCK_DATA (FILE_BLOCK_SIZE - sizeof(uint32) - sizeof(ObjRef) - sizeof(ObjRef))
 
 typedef struct
@@ -1446,7 +1545,18 @@ void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 
     check_object_header(ref, &header->object, header->common.header_chksum);
 
-    check_object_data(ref, (uint8*)object, size, header->common.data_chksum);
+#ifndef USE_PREDEFINED_STRUCTS
+    /* check to see if this is a file object (which has an older revision) */
+    if (is_same_object(&header->common.class_definition, get_file_001_reference()))
+    {
+	check_object_data(ref, (uint8*)object, sizeof(C_struct_File_001), header->common.data_chksum);
+	nwos_update_file_001_object_to_current(object, size);
+    }
+    else
+#endif
+    {
+	check_object_data(ref, (uint8*)object, size, header->common.data_chksum);
+    }
 }
 
 
@@ -1641,7 +1751,7 @@ void nwos_write_object_to_disk(ObjRef* ref, void* object, size_t size)
     size_t bytes_remaining;
     int i;
     int seq = 0;
-    uint8 block[FILE_BLOCK_SIZE];
+
 
     memset(ivec, 0, sizeof(ivec));
 
@@ -1652,7 +1762,11 @@ void nwos_write_object_to_disk(ObjRef* ref, void* object, size_t size)
     assert(nwos_block_used(ref));      /* assert bit in map was set by generate id */
 #endif
 
-    assert(!nwos_read_block(ref, block) || is_void_reference((ObjRef*)&block[1]));   /* make sure block is zero */
+#ifdef VERIFY_BLOCK_IS_EMPTY
+    /* NOTE: this was before using the bit map to verify that the block wasn't already */
+    /* used, now we assume the bit map is correct even if the block is non-zero */
+    assert(!nwos_read_block(ref, block) || is_void_reference((ObjRef*)&block[4]));   /* make sure block is zero */
+#endif
 
     ptr_obj = (uint8*) object;
 
@@ -1752,7 +1866,16 @@ void nwos_get_object_class(ObjRef* obj, ObjRef* object_class)
 	check_object_header(obj, &header.object, header.common.header_chksum);
     }
 
-    memcpy(object_class, &header.common.class_definition, sizeof(object_class));
+#ifndef USE_PREDEFINED_STRUCTS
+    if (is_same_object(&header.common.class_definition, get_file_001_reference()))
+    {
+	copy_reference(object_class, get_file_class_reference());
+    }
+    else
+#endif
+    {
+	copy_reference(object_class, &header.common.class_definition);
+    }
 }
 
 
