@@ -1,23 +1,42 @@
-/*             This file is part of the New World OS project
+/*
+--             This file is part of the New World OS project
 --                   Copyright (C) 2007  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
 --
--- NWOS is free software;  you can redistribute it and/or modify it under the
--- terms of the GNU General Public License  as published by the Free Software
--- Foundation; either version 2, or (at your option) any later version.  This
--- software is distributed with the hope that it will be useful,  but WITHOUT
--- ANY WARRANTY;  without  even the  implied warranty  of MERCHANTABILITY  or
--- FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License for
--- more  details.  You should have received a copy  of the GNU General Public
--- License along with this package;  see the file LICENSE.  If not, write to:
+--   This program is free software: you can redistribute it and/or modify
+--   it under the terms of the GNU General Public License as published by
+--   the Free Software Foundation, either version 3 of the License, or
+--   (at your option) any later version.
+--
+--   This program is distributed in the hope that it will be useful,
+--   but WITHOUT ANY WARRANTY; without even the implied warranty of
+--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+--   GNU General Public License for more details.
+--
+--   You should have received a copy of the GNU General Public License
+--   along with this program, in the file LICENSE.  If not, see 
+--   <http://www.gnu.org/licenses/>.
+--
+--   You can also contact me via paper mail at:
+--
+--      QRW Software
+--      P.O. Box 27511
+--      Salt Lake City, UT 84127-0511, USA.
 --
---      Free Software Foundation, Inc.
---      59 Temple Place - Suite 330
---      Boston, MA 02111-1307, USA.
 --
 -- $Log: verify_file.c,v $
+-- Revision 1.9  2007/07/21 14:57:01  jsedwards
+-- Added 'match' parameter in call to nwos_file_is_identical so it will ignore
+-- the time stamp on the file if desired.
+--
+-- Revision 1.8  2007/07/21 11:41:30  jsedwards
+-- Added option to read compressed file instead of default storage.
+--
+-- Revision 1.7  2007/07/01 19:44:12  jsedwards
+-- Upgrade to GPLv3.
+--
 -- Revision 1.6  2007/06/13 21:20:58  jsedwards
 -- Change to pass null to find_matching_path_and_file function for the
 -- new directory parameter (which makes it work the same as before).
@@ -61,18 +80,31 @@ int main(int argc, char* argv[])
     int i;
     bool do_remove = false;
     MatchCode match = MatchAll;
+    StorageType type = DEFAULT_TYPE_RO;
+    char* path = DEFAULT_FILE;
 
     printf("\n");
 
-    for (i = 1; i < argc; i++)
+    for (i = 1; i < argc && argv[i][0] == '-'; i++)
     {
-	if (argv[i][0] == '-')
+	if (strcmp(argv[i], "--remove") == 0)
 	{
-	    if (strcmp(argv[i], "--remove") != 0 && strcmp(argv[i], "--ignore-time") != 0)
-	    {
-		fprintf(stderr, "Unknown option: %s\n", argv[i]);
-		exit(1);
-	    }
+	    do_remove = true;
+	}
+	else if (strcmp(argv[i], "--ignore-time") == 0)
+	{
+	    match = IgnoreTime;
+	}
+	else if (strcmp(argv[i], "--compressed") == 0)
+	{
+	    type = Compressed_File_RO;
+	    i++;
+	    path = argv[i];
+	}
+	else
+	{
+	    fprintf(stderr, "Unknown option: %s\n", argv[i]);
+	    exit(1);
 	}
     }
 
@@ -85,69 +117,58 @@ int main(int argc, char* argv[])
     memcpy(root_object_ref.id, big_key+20, 4);
     serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
 
-    nwos_initialize_objectify(bf_key, linear, serial, DEFAULT_TYPE_RO, DEFAULT_FILE);
+    nwos_initialize_objectify(bf_key, linear, serial, type, path);
 
     nwos_set_root_object(&root_object_ref);
 
     nwos_set_encryption_level(Encryption_Low);
 
 
-    for (i = 1; i < argc; i++)
+    for (/* use i as left by loop above */; i < argc; i++)
     {
-	if (strcmp(argv[i], "--remove") == 0)
-	{
-	    do_remove = true;
-	}
-	else if (strcmp(argv[i], "--ignore-time") == 0)
-	{
-	    match = IgnoreTime;
-	}
-	else
-	{
-	    printf("%s: ", argv[i]);
-	    fflush(stdout);
+	printf("%s: ", argv[i]);
+	fflush(stdout);
 
-	    if (nwos_find_matching_path_and_file_association(NULL, argv[i], &ref, match))
+	if (nwos_find_matching_path_and_file_association(NULL, argv[i], &ref, match))
+	{
+	    if (do_remove)
 	    {
-		if (do_remove)
+		if (nwos_file_is_stored(&ref))
 		{
-		    if (nwos_file_is_stored(&ref))
-		    {
-			assert(nwos_file_is_identical(&ref, argv[i]));
+		    assert(nwos_file_is_identical(&ref, argv[i], match));
 
-			if (unlink(argv[i]) != 0)
-			{
-			    perror(argv[i]);
-			    exit(1);
-			}
-
-			printf("removed\n");
-		    }
-		    else if (nwos_file_has_backup(&ref))
+		    if (unlink(argv[i]) != 0)
 		    {
-			if (unlink(argv[i]) != 0)
-			{
-			    perror(argv[i]);
-			    exit(1);
-			}
+			perror(argv[i]);
+			exit(1);
+		    }
 
-			printf("removed\n");
-		    }
-		    else
+		    printf("removed\n");
+		}
+		else if (nwos_file_has_backup(&ref))
+		{
+		    if (unlink(argv[i]) != 0)
 		    {
-			printf("no known backup, not removed\n");
+			perror(argv[i]);
+			exit(1);
 		    }
+
+		    printf("removed\n");
 		}
 		else
 		{
-		    printf("found: %02x%02x%02x%02x\n", ref.id[0], ref.id[1], ref.id[2], ref.id[3]);
+		    printf("no known backup, not removed\n");
 		}
 	    }
 	    else
 	    {
-		printf("NOT found\n");
+		printf("found: %02x%02x%02x%02x\n", ref.id[0], ref.id[1], ref.id[2], ref.id[3]);
 	    }
 	}
+	else
+	{
+	    printf("NOT found\n");
+	}
     }
 
     nwos_terminate_objectify();
